package com.product.center.manage.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import com.product.center.manage.domain.CenterOrderMain;
import com.product.center.manage.domain.CenterOrderPerson;
import com.product.center.manage.domain.CenterPolicySynConf;
import com.product.center.manage.domain.CenterPolicySynLog;
import com.product.center.manage.mapper.CenterOrderMainMapper;
import com.product.center.manage.mapper.CenterOrderPersonMapper;
import com.product.center.manage.mapper.CenterPolicySynConfMapper;
import com.product.center.manage.mapper.CenterPolicySynLogMapper;
import com.product.center.manage.service.PolicySynService;
import com.product.center.manage.utils.AesUtil;
import com.product.center.manage.utils.SignUtil;
import com.product.center.manage.vo.CenterOrderMainVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ：fxl
 * @date ：Created in 2022/9/22 10:45
 * @description：
 */
@Repository
@Slf4j
public class PolicySynServiceImpl implements PolicySynService {

    @Autowired
    private CenterPolicySynConfMapper centerPolicySynConfMapper;

    @Autowired
    private CenterOrderMainMapper centerOrderMainMapper;


    @Autowired
    private CenterOrderPersonMapper centerOrderPersonMapper;

    @Autowired
    private CenterPolicySynLogMapper centerPolicySynLogMapper;

    /**
     * 开始同步保单数据
     */
    @Override
    public void startPolicySyn() {
        LambdaQueryWrapper<CenterPolicySynConf> lqw = Wrappers.lambdaQuery();
        lqw.eq(true,CenterPolicySynConf::getIsOpen,1);
        List<CenterPolicySynConf> centerPolicySynConfs = centerPolicySynConfMapper.selectList(lqw);
        for (CenterPolicySynConf centerPolicySynConf : centerPolicySynConfs) {
            // 同步保单
            synPolicy(centerPolicySynConf);
        }
    }

    /**
     * 同步保单逻辑
     * @param centerPolicySynConf
     */
    private void synPolicy(CenterPolicySynConf centerPolicySynConf) {
        // 根据配置获取保单
        List<CenterOrderMain> centerOrderMains = getpolicyList(centerPolicySynConf);
        for (CenterOrderMain centerOrderMain : centerOrderMains) {
            // 按照配置走不同请求
            commonReq(centerOrderMain,centerPolicySynConf);
        }


    }

    /**
     * 公共请求方法
     * @param centerOrderMain
     * @param centerPolicySynConf
     */
    private void commonReq(CenterOrderMain centerOrderMain, CenterPolicySynConf centerPolicySynConf) {
        CenterOrderMainVo centerOrderMainVo = new CenterOrderMainVo();
        BeanUtil.copyProperties(centerOrderMain,centerOrderMainVo);

        Map<String, String> jsonMap = JSON.parseObject(JSON.toJSONString(centerOrderMainVo), new TypeReference<Map<String, String>>() {});
        String sign = SignUtil.sign(jsonMap,centerPolicySynConf.getSignKey());
        jsonMap.put("sign", sign);

        String centerOrderMainJson = JSON.toJSONString(jsonMap);
        log.info("{}渠道保单同步请求明文:=> {}",centerPolicySynConf.getAliasName(),centerOrderMainJson);
        String req = AesUtil.encryptFromString(centerOrderMainJson, centerPolicySynConf.getAesKey());
        Map<String, String> reqMap = new HashMap<>();
        reqMap.put("data",req);
        String reqjson = JSON.toJSONString(reqMap);
        log.info("{}渠道保单同步请求密文:=> {}",centerPolicySynConf.getAliasName(),reqjson);
        // 发起请求
        String repsonResult = HttpUtil.post(centerPolicySynConf.getUri(), reqjson);
        log.info("{}渠道保单同步返回报文:=> {}",centerPolicySynConf.getAliasName(),repsonResult);
        JSONObject jsonObject = JSON.parseObject(repsonResult);
        if("0".equals(jsonObject.getString("code"))){
            CenterPolicySynLog centerPolicySynLog = CenterPolicySynLog.builder()
                    .confId(centerOrderMain.getId())
                    .orderNo(centerOrderMain.getOrderNo())
                    .channel(centerOrderMain.getChannel())
                    .synState(0)
                    .synFailHanState(0)
                    .resMsg(repsonResult).build();
            centerPolicySynLogMapper.insert(centerPolicySynLog);
        }else {
            CenterPolicySynLog centerPolicySynLog = CenterPolicySynLog.builder()
                    .confId(centerOrderMain.getId())
                    .orderNo(centerOrderMain.getOrderNo())
                    .channel(centerOrderMain.getChannel())
                    .synState(1)
                    .synFailHanState(0)
                    .resMsg(repsonResult).build();
            centerPolicySynLogMapper.insert(centerPolicySynLog);
        }
        centerPolicySynConf.setLastSynPolicyId(centerOrderMain.getTriggerId());
        centerPolicySynConfMapper.updateById(centerPolicySynConf);
    }


    /**
     * 根据条件查询保单数据
     */
    public List<CenterOrderMain> getpolicyList(CenterPolicySynConf centerPolicySynConf){
        LambdaQueryWrapper<CenterOrderMain> lqw = Wrappers.lambdaQuery();
        lqw.eq(true,CenterOrderMain::getChannel,centerPolicySynConf.getChannel());
        lqw.gt(true,CenterOrderMain::getTriggerId,centerPolicySynConf.getLastSynPolicyId());
        lqw.orderByAsc(CenterOrderMain::getTriggerId);
        lqw.last(" limit 100");
        // 查询保单客户
        List<CenterOrderMain> centerOrderMains = centerOrderMainMapper.selectList(lqw);
        if(centerOrderMains != null){
            // 提取保单 订单号
            List<String> orderNoList = centerOrderMains.stream()
                    .map(CenterOrderMain::getOrderNo)
                    .collect(Collectors.toList());

            LambdaQueryWrapper<CenterOrderPerson> prLqw = Wrappers.lambdaQuery();
            prLqw.in(true,CenterOrderPerson::getOrderNo,orderNoList);
            // 根据订单号获取客户信息
            List<CenterOrderPerson> centerOrderPeople = centerOrderPersonMapper.selectList(prLqw);
            // 获取的客户信息 按照订单号分组
            Map<String, List<CenterOrderPerson>> prOrderNoMap = centerOrderPeople.stream()
                    .collect(Collectors.groupingBy(CenterOrderPerson::getOrderNo));
            // 将客户信息与保单绑定起来
            centerOrderMains.forEach(e -> {
                List<CenterOrderPerson> centerOrderPeopleList = prOrderNoMap.get(e.getOrderNo());
                e.setInsureds(centerOrderPeopleList);
            });
        }

        return centerOrderMains;
    }





}
